<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="utf-8" />
  <meta http-equiv="X-UA-Compatible" content="IE=edge" />
  <meta name="viewport" content="width=device-width, initial-scale=1" />
  <meta name="csrf-token" content="{{ csrf_token }}" />
  <link href="/assets/app.css" rel="stylesheet" type="text/css" />
  <title>
    {% block title %}React.js page{% endblock%}
  </title>
  {% block css %}{% endblock %}
</head>

<body>
  <div id="app"></div>
  <script src="/js/app.js" defer></script>
</body>

</html>